/* 
 * VirtualBox Control Service
 * 
 * Copyright © 2010 vu1tur.eu.org
 * 
 * This file is part of VirtualBox Control Service
 * 
 * VirtualBox Control Service is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 * 
 * VirtualBox Control Service is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
#include "vboxipcc.h"
#include "vboxctrltray.h"
#include <stdio.h>
 
BOOL VBoxIPCC_GetVRDPEnabled(const char *vmname)
{
	char pipename[256];
	if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);

	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return FALSE;
	DWORD enabled = 0;
	DWORD dwFunc = VBOX_IPC_C_GETVRDPENABLED;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &enabled, sizeof(enabled), &cbBytes, NULL);
	CloseHandle(hPipe);
	
	return (BOOL)enabled;
}
 
LONG VBoxIPCC_GetVRDPPort(const char *vmname)
{
	char pipename[256];
	if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);
	
	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return -1;
	DWORD port = 0;
	DWORD dwFunc = VBOX_IPC_C_GETVRDPPORT;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &port, sizeof(port), &cbBytes, NULL);
	CloseHandle(hPipe);
	
	return (LONG)port;
}
 
BOOL VBoxIPCC_GetVRDPResolution(const char *vmname, LONG *width, LONG *height)
{
	 char pipename[256];
	 if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		 return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);
	
	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return -1;
	DWORD resolution;
	DWORD dwFunc = VBOX_IPC_C_GETVRDPRESOLUTION;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &resolution, sizeof(resolution), &cbBytes, NULL);
	CloseHandle(hPipe);
	*width = LOWORD(resolution);
	*height = HIWORD(resolution);
	
	return TRUE;
}

LONG VBoxIPCC_GetState(const char *vmname)
{
	DWORD status = -1;
	char pipename[256];
	if (strlen(vmname) + strlen(VBOX_PIPENAME_BASE) > 255)
		return FALSE;
	sprintf(pipename, "%s.%s", VBOX_PIPENAME_BASE, vmname);
	
	DWORD cbBytes;
	HANDLE hPipe = CreateFile(pipename, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
	if (hPipe == INVALID_HANDLE_VALUE)
		return -1;
	DWORD dwFunc = VBOX_IPC_C_GETSTATE;
	BOOL bResult = WriteFile(hPipe, &dwFunc, sizeof(dwFunc), &cbBytes, NULL);
	if (bResult)
		ReadFile(hPipe, &status, sizeof(status), &cbBytes, NULL);
	CloseHandle(hPipe);
	
	return (LONG)status;
}

BOOL fIPCListenerRunning;

void VBoxTrayIPCListener(void* args)
{
	UNUSED(args);
	PSECURITY_DESCRIPTOR psd;
	PACL pacl;
	EXPLICIT_ACCESS ea[2];
	SECURITY_ATTRIBUTES sa;
	
	BuildExplicitAccessWithName(ea, TEXT("INTERACTIVE"), ACCESS_SYSTEM_SECURITY | FILE_GENERIC_READ | FILE_GENERIC_WRITE, GRANT_ACCESS, NO_INHERITANCE);
	BuildExplicitAccessWithName(ea+1, TEXT("SYSTEM"), ACCESS_SYSTEM_SECURITY | FILE_GENERIC_READ | FILE_GENERIC_WRITE, GRANT_ACCESS, NO_INHERITANCE);
	SetEntriesInAcl(2, ea, NULL, &pacl);
	psd = (PSECURITY_DESCRIPTOR)LocalAlloc(LPTR, SECURITY_DESCRIPTOR_MIN_LENGTH);
	InitializeSecurityDescriptor(psd, SECURITY_DESCRIPTOR_REVISION);
	SetSecurityDescriptorDacl(psd, TRUE, pacl, FALSE);
	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
	sa.lpSecurityDescriptor = psd;
	sa.bInheritHandle = FALSE;
	
	HANDLE hPipe = INVALID_HANDLE_VALUE;	
	
	fIPCListenerRunning = TRUE;
	while (fIPCListenerRunning)
	{
		hPipe = CreateNamedPipe(
			VBOXTRAY_PIPENAME,
			PIPE_ACCESS_DUPLEX,	
			PIPE_TYPE_BYTE | PIPE_READMODE_BYTE | PIPE_WAIT,
			PIPE_UNLIMITED_INSTANCES,
			16,
			16,
			NMPWAIT_USE_DEFAULT_WAIT,
			&sa);
		
		if (hPipe == INVALID_HANDLE_VALUE)
			break;

		BOOL fConnected = ConnectNamedPipe(hPipe, NULL) ? TRUE : GetLastError() == ERROR_PIPE_CONNECTED;
	
		if (fConnected)
			_beginthread(VBoxTrayIPCDispatcher, 0, (void*)hPipe);
		else
			CloseHandle(hPipe);
	}
	LocalFree(pacl);
	LocalFree(psd);
}

void VBoxTrayIPCDispatcher(HANDLE hp)
{
	if (hp != INVALID_HANDLE_VALUE)
	{
		DWORD dwBuffer;
		DWORD cbBytes;
			
		BOOL bResult = ReadFile(hp, (void*)&dwBuffer, sizeof(dwBuffer), &cbBytes, NULL);
		if (!bResult || (cbBytes == 0))
		{
			//Log(1, "ERROR reading from client");
		} else {
			switch (dwBuffer)
			{
				case VBOX_IPC_C_RELOAD:
					VBoxTrayIPCReload();
					break;
				default:
					break;
			}
		}
		FlushFileBuffers(hp);
		DisconnectNamedPipe(hp);
		CloseHandle(hp);
	}
}

void VBoxTrayIPCReload()
{
	UpdateServices(true);
}